from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_info_response_200_allowed_teams_cumulative_item import (
        DatasetInfoResponse200AllowedTeamsCumulativeItem,
    )
    from ..models.dataset_info_response_200_allowed_teams_item import (
        DatasetInfoResponse200AllowedTeamsItem,
    )
    from ..models.dataset_info_response_200_data_source import (
        DatasetInfoResponse200DataSource,
    )
    from ..models.dataset_info_response_200_data_store import (
        DatasetInfoResponse200DataStore,
    )


T = TypeVar("T", bound="DatasetInfoResponse200")


@attr.s(auto_attribs=True)
class DatasetInfoResponse200:
    """
    Attributes:
        name (str):
        data_source (DatasetInfoResponse200DataSource):
        data_store (DatasetInfoResponse200DataStore):
        allowed_teams (List['DatasetInfoResponse200AllowedTeamsItem']):
        is_active (int):
        is_public (int):
        description (str):
        display_name (str):
        created (int):
        tags (List[Any]):
        folder_id (str):
        owning_organization (Union[Unset, str]):
        allowed_teams_cumulative (Union[Unset, List['DatasetInfoResponse200AllowedTeamsCumulativeItem']]):
        is_editable (Union[Unset, int]):
        last_used_by_user (Union[Unset, int]):
        logo_url (Union[Unset, str]):
        sorting_key (Union[Unset, int]):
        details (Union[Unset, str]):
        is_unreported (Union[Unset, int]):
        jobs_enabled (Union[Unset, int]):
        publication (Union[Unset, str]):
    """

    name: str
    data_source: "DatasetInfoResponse200DataSource"
    data_store: "DatasetInfoResponse200DataStore"
    allowed_teams: List["DatasetInfoResponse200AllowedTeamsItem"]
    is_active: int
    is_public: int
    description: str
    display_name: str
    created: int
    tags: List[Any]
    folder_id: str
    owning_organization: Union[Unset, str] = UNSET
    allowed_teams_cumulative: Union[
        Unset, List["DatasetInfoResponse200AllowedTeamsCumulativeItem"]
    ] = UNSET
    is_editable: Union[Unset, int] = UNSET
    last_used_by_user: Union[Unset, int] = UNSET
    logo_url: Union[Unset, str] = UNSET
    sorting_key: Union[Unset, int] = UNSET
    details: Union[Unset, str] = UNSET
    is_unreported: Union[Unset, int] = UNSET
    jobs_enabled: Union[Unset, int] = UNSET
    publication: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        data_source = self.data_source.to_dict()

        data_store = self.data_store.to_dict()

        allowed_teams = []
        for allowed_teams_item_data in self.allowed_teams:
            allowed_teams_item = allowed_teams_item_data.to_dict()

            allowed_teams.append(allowed_teams_item)

        is_active = self.is_active
        is_public = self.is_public
        description = self.description
        display_name = self.display_name
        created = self.created
        tags = self.tags

        folder_id = self.folder_id
        owning_organization = self.owning_organization
        allowed_teams_cumulative: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.allowed_teams_cumulative, Unset):
            allowed_teams_cumulative = []
            for allowed_teams_cumulative_item_data in self.allowed_teams_cumulative:
                allowed_teams_cumulative_item = (
                    allowed_teams_cumulative_item_data.to_dict()
                )

                allowed_teams_cumulative.append(allowed_teams_cumulative_item)

        is_editable = self.is_editable
        last_used_by_user = self.last_used_by_user
        logo_url = self.logo_url
        sorting_key = self.sorting_key
        details = self.details
        is_unreported = self.is_unreported
        jobs_enabled = self.jobs_enabled
        publication = self.publication

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "dataSource": data_source,
                "dataStore": data_store,
                "allowedTeams": allowed_teams,
                "isActive": is_active,
                "isPublic": is_public,
                "description": description,
                "displayName": display_name,
                "created": created,
                "tags": tags,
                "folderId": folder_id,
            }
        )
        if owning_organization is not UNSET:
            field_dict["owningOrganization"] = owning_organization
        if allowed_teams_cumulative is not UNSET:
            field_dict["allowedTeamsCumulative"] = allowed_teams_cumulative
        if is_editable is not UNSET:
            field_dict["isEditable"] = is_editable
        if last_used_by_user is not UNSET:
            field_dict["lastUsedByUser"] = last_used_by_user
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url
        if sorting_key is not UNSET:
            field_dict["sortingKey"] = sorting_key
        if details is not UNSET:
            field_dict["details"] = details
        if is_unreported is not UNSET:
            field_dict["isUnreported"] = is_unreported
        if jobs_enabled is not UNSET:
            field_dict["jobsEnabled"] = jobs_enabled
        if publication is not UNSET:
            field_dict["publication"] = publication

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dataset_info_response_200_allowed_teams_cumulative_item import (
            DatasetInfoResponse200AllowedTeamsCumulativeItem,
        )
        from ..models.dataset_info_response_200_allowed_teams_item import (
            DatasetInfoResponse200AllowedTeamsItem,
        )
        from ..models.dataset_info_response_200_data_source import (
            DatasetInfoResponse200DataSource,
        )
        from ..models.dataset_info_response_200_data_store import (
            DatasetInfoResponse200DataStore,
        )

        d = src_dict.copy()
        name = d.pop("name")

        data_source = DatasetInfoResponse200DataSource.from_dict(d.pop("dataSource"))

        data_store = DatasetInfoResponse200DataStore.from_dict(d.pop("dataStore"))

        allowed_teams = []
        _allowed_teams = d.pop("allowedTeams")
        for allowed_teams_item_data in _allowed_teams:
            allowed_teams_item = DatasetInfoResponse200AllowedTeamsItem.from_dict(
                allowed_teams_item_data
            )

            allowed_teams.append(allowed_teams_item)

        is_active = d.pop("isActive")

        is_public = d.pop("isPublic")

        description = d.pop("description")

        display_name = d.pop("displayName")

        created = d.pop("created")

        tags = cast(List[Any], d.pop("tags"))

        folder_id = d.pop("folderId")

        owning_organization = d.pop("owningOrganization", UNSET)

        allowed_teams_cumulative = []
        _allowed_teams_cumulative = d.pop("allowedTeamsCumulative", UNSET)
        for allowed_teams_cumulative_item_data in _allowed_teams_cumulative or []:
            allowed_teams_cumulative_item = (
                DatasetInfoResponse200AllowedTeamsCumulativeItem.from_dict(
                    allowed_teams_cumulative_item_data
                )
            )

            allowed_teams_cumulative.append(allowed_teams_cumulative_item)

        is_editable = d.pop("isEditable", UNSET)

        last_used_by_user = d.pop("lastUsedByUser", UNSET)

        logo_url = d.pop("logoUrl", UNSET)

        sorting_key = d.pop("sortingKey", UNSET)

        details = d.pop("details", UNSET)

        is_unreported = d.pop("isUnreported", UNSET)

        jobs_enabled = d.pop("jobsEnabled", UNSET)

        publication = d.pop("publication", UNSET)

        dataset_info_response_200 = cls(
            name=name,
            data_source=data_source,
            data_store=data_store,
            allowed_teams=allowed_teams,
            is_active=is_active,
            is_public=is_public,
            description=description,
            display_name=display_name,
            created=created,
            tags=tags,
            folder_id=folder_id,
            owning_organization=owning_organization,
            allowed_teams_cumulative=allowed_teams_cumulative,
            is_editable=is_editable,
            last_used_by_user=last_used_by_user,
            logo_url=logo_url,
            sorting_key=sorting_key,
            details=details,
            is_unreported=is_unreported,
            jobs_enabled=jobs_enabled,
            publication=publication,
        )

        dataset_info_response_200.additional_properties = d
        return dataset_info_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.build_info_response_200_webknossos import (
        BuildInfoResponse200Webknossos,
    )
    from ..models.build_info_response_200_webknossos_wrap import (
        BuildInfoResponse200WebknossosWrap,
    )


T = TypeVar("T", bound="BuildInfoResponse200")


@attr.s(auto_attribs=True)
class BuildInfoResponse200:
    """
    Attributes:
        webknossos (Union[Unset, BuildInfoResponse200Webknossos]):
        webknossos_wrap (Union[Unset, BuildInfoResponse200WebknossosWrap]):
        schema_version (Union[Unset, int]):
        local_data_store_enabled (Union[Unset, int]):
        local_tracing_store_enabled (Union[Unset, int]):
    """

    webknossos: Union[Unset, "BuildInfoResponse200Webknossos"] = UNSET
    webknossos_wrap: Union[Unset, "BuildInfoResponse200WebknossosWrap"] = UNSET
    schema_version: Union[Unset, int] = UNSET
    local_data_store_enabled: Union[Unset, int] = UNSET
    local_tracing_store_enabled: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        webknossos: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.webknossos, Unset):
            webknossos = self.webknossos.to_dict()

        webknossos_wrap: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.webknossos_wrap, Unset):
            webknossos_wrap = self.webknossos_wrap.to_dict()

        schema_version = self.schema_version
        local_data_store_enabled = self.local_data_store_enabled
        local_tracing_store_enabled = self.local_tracing_store_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if webknossos is not UNSET:
            field_dict["webknossos"] = webknossos
        if webknossos_wrap is not UNSET:
            field_dict["webknossos-wrap"] = webknossos_wrap
        if schema_version is not UNSET:
            field_dict["schemaVersion"] = schema_version
        if local_data_store_enabled is not UNSET:
            field_dict["localDataStoreEnabled"] = local_data_store_enabled
        if local_tracing_store_enabled is not UNSET:
            field_dict["localTracingStoreEnabled"] = local_tracing_store_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.build_info_response_200_webknossos import (
            BuildInfoResponse200Webknossos,
        )
        from ..models.build_info_response_200_webknossos_wrap import (
            BuildInfoResponse200WebknossosWrap,
        )

        d = src_dict.copy()
        _webknossos = d.pop("webknossos", UNSET)
        webknossos: Union[Unset, BuildInfoResponse200Webknossos]
        if isinstance(_webknossos, Unset):
            webknossos = UNSET
        else:
            webknossos = BuildInfoResponse200Webknossos.from_dict(_webknossos)

        _webknossos_wrap = d.pop("webknossos-wrap", UNSET)
        webknossos_wrap: Union[Unset, BuildInfoResponse200WebknossosWrap]
        if isinstance(_webknossos_wrap, Unset):
            webknossos_wrap = UNSET
        else:
            webknossos_wrap = BuildInfoResponse200WebknossosWrap.from_dict(
                _webknossos_wrap
            )

        schema_version = d.pop("schemaVersion", UNSET)

        local_data_store_enabled = d.pop("localDataStoreEnabled", UNSET)

        local_tracing_store_enabled = d.pop("localTracingStoreEnabled", UNSET)

        build_info_response_200 = cls(
            webknossos=webknossos,
            webknossos_wrap=webknossos_wrap,
            schema_version=schema_version,
            local_data_store_enabled=local_data_store_enabled,
            local_tracing_store_enabled=local_tracing_store_enabled,
        )

        build_info_response_200.additional_properties = d
        return build_info_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_info_response_200_task_type_settings import (
        AnnotationInfoResponse200TaskTypeSettings,
    )


T = TypeVar("T", bound="AnnotationInfoResponse200TaskType")


@attr.s(auto_attribs=True)
class AnnotationInfoResponse200TaskType:
    """
    Attributes:
        id (str):
        summary (str):
        description (str):
        team_id (str):
        team_name (str):
        settings (Union[Unset, AnnotationInfoResponse200TaskTypeSettings]):
        recommended_configuration (Union[Unset, str]):
        tracing_type (Union[Unset, str]):
    """

    id: str
    summary: str
    description: str
    team_id: str
    team_name: str
    settings: Union[Unset, "AnnotationInfoResponse200TaskTypeSettings"] = UNSET
    recommended_configuration: Union[Unset, str] = UNSET
    tracing_type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        summary = self.summary
        description = self.description
        team_id = self.team_id
        team_name = self.team_name
        settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        recommended_configuration = self.recommended_configuration
        tracing_type = self.tracing_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "summary": summary,
                "description": description,
                "teamId": team_id,
                "teamName": team_name,
            }
        )
        if settings is not UNSET:
            field_dict["settings"] = settings
        if recommended_configuration is not UNSET:
            field_dict["recommendedConfiguration"] = recommended_configuration
        if tracing_type is not UNSET:
            field_dict["tracingType"] = tracing_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_info_response_200_task_type_settings import (
            AnnotationInfoResponse200TaskTypeSettings,
        )

        d = src_dict.copy()
        id = d.pop("id")

        summary = d.pop("summary")

        description = d.pop("description")

        team_id = d.pop("teamId")

        team_name = d.pop("teamName")

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, AnnotationInfoResponse200TaskTypeSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = AnnotationInfoResponse200TaskTypeSettings.from_dict(_settings)

        recommended_configuration = d.pop("recommendedConfiguration", UNSET)

        tracing_type = d.pop("tracingType", UNSET)

        annotation_info_response_200_task_type = cls(
            id=id,
            summary=summary,
            description=description,
            team_id=team_id,
            team_name=team_name,
            settings=settings,
            recommended_configuration=recommended_configuration,
            tracing_type=tracing_type,
        )

        annotation_info_response_200_task_type.additional_properties = d
        return annotation_info_response_200_task_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.user_logged_time_response_200_logged_time_item import (
        UserLoggedTimeResponse200LoggedTimeItem,
    )


T = TypeVar("T", bound="UserLoggedTimeResponse200")


@attr.s(auto_attribs=True)
class UserLoggedTimeResponse200:
    """
    Attributes:
        logged_time (List['UserLoggedTimeResponse200LoggedTimeItem']):
    """

    logged_time: List["UserLoggedTimeResponse200LoggedTimeItem"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        logged_time = []
        for logged_time_item_data in self.logged_time:
            logged_time_item = logged_time_item_data.to_dict()

            logged_time.append(logged_time_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "loggedTime": logged_time,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_logged_time_response_200_logged_time_item import (
            UserLoggedTimeResponse200LoggedTimeItem,
        )

        d = src_dict.copy()
        logged_time = []
        _logged_time = d.pop("loggedTime")
        for logged_time_item_data in _logged_time:
            logged_time_item = UserLoggedTimeResponse200LoggedTimeItem.from_dict(
                logged_time_item_data
            )

            logged_time.append(logged_time_item)

        user_logged_time_response_200 = cls(
            logged_time=logged_time,
        )

        user_logged_time_response_200.additional_properties = d
        return user_logged_time_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_info_response_200_data_source_data_layers_item_bounding_box import (
        DatasetInfoResponse200DataSourceDataLayersItemBoundingBox,
    )
    from ..models.dataset_info_response_200_data_source_data_layers_item_default_view_configuration import (
        DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration,
    )


T = TypeVar("T", bound="DatasetInfoResponse200DataSourceDataLayersItem")


@attr.s(auto_attribs=True)
class DatasetInfoResponse200DataSourceDataLayersItem:
    """
    Attributes:
        name (str):
        category (str):
        bounding_box (DatasetInfoResponse200DataSourceDataLayersItemBoundingBox):
        resolutions (List[List[int]]):
        element_class (str):
        default_view_configuration (Union[Unset,
            DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration]):
    """

    name: str
    category: str
    bounding_box: "DatasetInfoResponse200DataSourceDataLayersItemBoundingBox"
    resolutions: List[List[int]]
    element_class: str
    default_view_configuration: Union[
        Unset, "DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration"
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        category = self.category
        bounding_box = self.bounding_box.to_dict()

        resolutions = []
        for resolutions_item_data in self.resolutions:
            resolutions_item = resolutions_item_data

            resolutions.append(resolutions_item)

        element_class = self.element_class
        default_view_configuration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_view_configuration, Unset):
            default_view_configuration = self.default_view_configuration.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "category": category,
                "boundingBox": bounding_box,
                "resolutions": resolutions,
                "elementClass": element_class,
            }
        )
        if default_view_configuration is not UNSET:
            field_dict["defaultViewConfiguration"] = default_view_configuration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dataset_info_response_200_data_source_data_layers_item_bounding_box import (
            DatasetInfoResponse200DataSourceDataLayersItemBoundingBox,
        )
        from ..models.dataset_info_response_200_data_source_data_layers_item_default_view_configuration import (
            DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration,
        )

        d = src_dict.copy()
        name = d.pop("name")

        category = d.pop("category")

        bounding_box = (
            DatasetInfoResponse200DataSourceDataLayersItemBoundingBox.from_dict(
                d.pop("boundingBox")
            )
        )

        resolutions = []
        _resolutions = d.pop("resolutions")
        for resolutions_item_data in _resolutions:
            resolutions_item = cast(List[int], resolutions_item_data)

            resolutions.append(resolutions_item)

        element_class = d.pop("elementClass")

        _default_view_configuration = d.pop("defaultViewConfiguration", UNSET)
        default_view_configuration: Union[
            Unset,
            DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration,
        ]
        if isinstance(_default_view_configuration, Unset):
            default_view_configuration = UNSET
        else:
            default_view_configuration = DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration.from_dict(
                _default_view_configuration
            )

        dataset_info_response_200_data_source_data_layers_item = cls(
            name=name,
            category=category,
            bounding_box=bounding_box,
            resolutions=resolutions,
            element_class=element_class,
            default_view_configuration=default_view_configuration,
        )

        dataset_info_response_200_data_source_data_layers_item.additional_properties = d
        return dataset_info_response_200_data_source_data_layers_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.js_object_underlying import JsObjectUnderlying
    from ..models.js_object_value import JsObjectValue
    from ..models.tuple_2_string_js_value import Tuple2StringJsValue


T = TypeVar("T", bound="JsObject")


@attr.s(auto_attribs=True)
class JsObject:
    """
    Attributes:
        underlying (JsObjectUnderlying):
        fields (List['Tuple2StringJsValue']):
        value (JsObjectValue):
    """

    underlying: "JsObjectUnderlying"
    fields: List["Tuple2StringJsValue"]
    value: "JsObjectValue"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        underlying = self.underlying.to_dict()

        fields = []
        for fields_item_data in self.fields:
            fields_item = fields_item_data.to_dict()

            fields.append(fields_item)

        value = self.value.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "underlying": underlying,
                "fields": fields,
                "value": value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.js_object_underlying import JsObjectUnderlying
        from ..models.js_object_value import JsObjectValue
        from ..models.tuple_2_string_js_value import Tuple2StringJsValue

        d = src_dict.copy()
        underlying = JsObjectUnderlying.from_dict(d.pop("underlying"))

        fields = []
        _fields = d.pop("fields")
        for fields_item_data in _fields:
            fields_item = Tuple2StringJsValue.from_dict(fields_item_data)

            fields.append(fields_item)

        value = JsObjectValue.from_dict(d.pop("value"))

        js_object = cls(
            underlying=underlying,
            fields=fields,
            value=value,
        )

        js_object.additional_properties = d
        return js_object

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.linked_layer_identifier import LinkedLayerIdentifier


T = TypeVar("T", bound="ReserveUploadInformation")


@attr.s(auto_attribs=True)
class ReserveUploadInformation:
    """
    Attributes:
        upload_id (str):
        name (str):
        organization (str):
        total_file_count (int):
        initial_teams (List[str]):
        layers_to_link (Union[Unset, List['LinkedLayerIdentifier']]):
        folder_id (Union[Unset, str]):
    """

    upload_id: str
    name: str
    organization: str
    total_file_count: int
    initial_teams: List[str]
    layers_to_link: Union[Unset, List["LinkedLayerIdentifier"]] = UNSET
    folder_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        upload_id = self.upload_id
        name = self.name
        organization = self.organization
        total_file_count = self.total_file_count
        initial_teams = self.initial_teams

        layers_to_link: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.layers_to_link, Unset):
            layers_to_link = []
            for layers_to_link_item_data in self.layers_to_link:
                layers_to_link_item = layers_to_link_item_data.to_dict()

                layers_to_link.append(layers_to_link_item)

        folder_id = self.folder_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uploadId": upload_id,
                "name": name,
                "organization": organization,
                "totalFileCount": total_file_count,
                "initialTeams": initial_teams,
            }
        )
        if layers_to_link is not UNSET:
            field_dict["layersToLink"] = layers_to_link
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.linked_layer_identifier import LinkedLayerIdentifier

        d = src_dict.copy()
        upload_id = d.pop("uploadId")

        name = d.pop("name")

        organization = d.pop("organization")

        total_file_count = d.pop("totalFileCount")

        initial_teams = cast(List[str], d.pop("initialTeams"))

        layers_to_link = []
        _layers_to_link = d.pop("layersToLink", UNSET)
        for layers_to_link_item_data in _layers_to_link or []:
            layers_to_link_item = LinkedLayerIdentifier.from_dict(
                layers_to_link_item_data
            )

            layers_to_link.append(layers_to_link_item)

        folder_id = d.pop("folderId", UNSET)

        reserve_upload_information = cls(
            upload_id=upload_id,
            name=name,
            organization=organization,
            total_file_count=total_file_count,
            initial_teams=initial_teams,
            layers_to_link=layers_to_link,
            folder_id=folder_id,
        )

        reserve_upload_information.additional_properties = d
        return reserve_upload_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
